#include "filed.h"
#include "dbq.h"
#include "log.h"
#include "main.h"
#include "externs.h"
#include "socket.h"
#include <string.h>
#include <malloc.h>

void dbq_process(void) {
	struct db *d;

	d = filed->db;
	
	if(shmem_db_free() < 8) dlog(DEBUG_IPC,"low on free shared memory segments: %d",shmem_db_free());

	while(d && (shmem_db_free() > 8)) {
		if(d->flags & DBFLAG_PROCESSING) {
			d = d->next;
			continue;
		}

		if(d->filter > 0) 
			if(!shmem_db_id_present(d->id))	
				shmem_db_ins(d);
		if(d->filter < 0)
			shmem_db_ins(d);
			
		d = d->next;
	}

	dbq_handle();
}

void dbq(int filter, int id, char *query) {
	struct db *d;
	struct db *e;

	/* first we malloc out a chunk of memory */

	dlog(DEBUG_DB,"dbq(): inserting filter %d id %d query [%s]",filter,id,query);
	if(!(d = (struct db *) malloc(sizeof(struct db)))) {
		elog("dbq(): unable to malloc");
		die();
	}

	d->query = strdup(query);
	d->id = id;
	d->filter = filter;
	d->seq = dbq_get_seq();
	d->loc = -1;
	d->output = NULL;
	d->flags = 0;

	if(!(e = dbq_get_tail())) {
		filed->db = d;
		d->prev = NULL;
		d->next = NULL;
		return;
	}	

	d->prev = e;
	e->next = d;
	d->next = NULL;
}

struct db *dbq_get_tail(void) {
	struct db *d = filed->db;

	if(!d) return(NULL);
	
	while(d->next) d = d->next;

	return(d);
}

int dbq_get_seq(void) {
	int ret = filed->dbid++;
	if(ret > 65534) filed->dbid = 0;
	return(ret);
}

void dbq_handle(void) {
	/* handle the processing of return data */
	/* primarily a dispatch function */
	
	struct db *db;

	while((db = shmem_db_get_done())) {
		c_send_to(db->id,db->output);
		if(db && (db->flags & DBFLAG_DONE)) dbq_rm(db);
	}

	/* 
	 * note that get done only clears the claim bit 
	 * if there isnt more data coming
	 */

	return;
}

void dbq_rm(struct db *d) {
	struct db *e;
	if(!d) return;
	
	e = filed->db;
	
	if(d == e) {
		if(e->next) {
			filed->db = e->next;
			e->prev = NULL;
		} else {
			filed->db = NULL;
		}
		
		if(e->query) free(e->query);
		if(e->output) free(e->output);
		free(e);
		return;
	}

	if(d->next) d->next->prev = d->prev;
	if(d->prev) d->prev->next = d->next;
	
	if(d->query) free(d->query);
	if(d->output) free(d->output);
	if(d) free(d); 
	/* previously, there was a bug which was free(e), which of course
	was the raver's dream. Fuck */
}

struct db *dbq_get_slot(int i) {
	struct db *ret = filed->db;

	while(ret) {
		if(ret->loc == i) return(ret);
		ret = ret->next;
	}
	return(NULL);
}

