#include "session.h"
#include "log.h"
#include "filed.h"
#include "externs.h"
#include "main.h"
#include "parse.h"
#include <string.h>
#include <malloc.h>

struct session *get_session(char *id) {
	struct session *s = get_session_i(id);

	if(!s)
		return(put_session(id));
	else return(s);
}

struct session *get_session_i(char *id) {
	struct session *s = filed->session;

	while(s) {
		if(s->session_id && !strcmp(s->session_id,id))
			return(s);
		s = s->next;
	}

	return(NULL);
}

struct session *put_session(char *id) {
	struct session *s;
	struct session *t;

	if(!(s = (struct session *) malloc(sizeof(struct session)))) {
		elog("Error: unable to malloc out memory for new session!");
		die();
	}

	s->session_seq = get_session_seq();

	t = filed->session;
	if(!t) {
		filed->session = s;
		} else {

		while(t->next) t = t->next;
		t->next = s;
		}
	s->next = NULL;
	s->create = time(NULL);
	s->last_access = s->create;
	s->prev = t;
	s->flags = 0;
	s->session_id = strdup(id);

	return(s);
}
	
struct session *get_session_by_id(int i) {
	/* unlike get_session, this returns NULL if the ID doesn't exist */
	struct session *s;

	s = filed->session;

	while(s) {
		if(s->session_seq == i) return(s);
		s = s->next;
	}

	return(NULL);
}

void rm_session(char *id) {
	struct session *s;
	struct session *t;
	if(!id) return;
	
	s = get_session_i(id);
	
	if(!s) return;

	shmem_session_kill(s->session_seq);

	t = filed->session;
	
	if(s == t) {
		/* special case */
		if(t->next) { 
			filed->session = t->next;
			t->next->prev = NULL;
		} else {
			filed->session = NULL;
		}

		free_session(t);
		return;
	}

	t = s;

	if(t->next) t->next->prev = t->prev;
	if(t->prev) t->prev->next = t->next;
	free_session(t);
	return;
}

	
void free_session(struct session *s) {
	if(!s) return;
	if(s->session_id) free(s->session_id);
	free(s);
}
	
int get_session_seq(void) {
	int ret;
	ret = ++filed->session_id;
	if(filed->session_id > 65534) filed->session_id = 0;
	return(ret);
}

	
void session_update(int session) {
	struct session *s;
	s = get_session_by_id(session);

	if(!s) return;
	s->last_access = time(NULL);
	return;
}

void session_do_timeouts(void) {
	struct session *s;
	struct session *r;
	time_t now = time(NULL);
	int timeout = get_one_data_int(filed->cf,"session-timeout");
	
	if(timeout < 1) return;

	s = filed->session;

	

	while(s) {
		r = s->next;
		if((s->last_access + timeout) < now) {
			dlog(DEBUG_FILE,"session %d timed out",s->session_seq);
			rm_session(s->session_id);
		}

		s = r;
	}
	return;
}
