#include "sys.h"
#include "filed.h"
#include "main.h"
#include "externs.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

int test_open(char *filename) {
	int fd;

	if((fd = open(filename,O_RDONLY + O_NONBLOCK)) == -1) {
		return(0);
	}

	close(fd);
	return(1);
}

int get_shmmax(void) {
	char inbuf[128];
	FILE *fp;

	if(!(fp = fopen("/proc/sys/kernel/shmmax","r")))
		return(0);

	fgets(inbuf,128,fp);
	fclose(fp);

	if(!inbuf) return(-1);
	return(atoi(inbuf));
}

int put_shmmax(int need) {
	char outbuf[128];
	FILE *fp;

	if(!(fp = fopen("/proc/sys/kernel/shmmax","w")))
		return(0);
	
	sprintf(outbuf,"%d\n",need);

	fputs(outbuf,fp);
	fclose(fp);

	if(get_shmmax() != need) return(0);
	return(1);
}
	
int get_avail_shmem(void) {
	char inbuf[128];
	int inuse;
	FILE *fp;

	if(!(fp = fopen("/proc/sys/kernel/shmmax","r")))
		return(0);

	fgets(inbuf,128,fp);
	fclose(fp);
	
	if(!inbuf) return(-1);
	inuse = atoi(inbuf);
	
	if(!(fp = fopen("/proc/sys/kernel/shmall","r")))
		return(0);

	fgets(inbuf,128,fp);
	fclose(fp);

	if(!inbuf) return(-1);
	return(inuse - atoi(inbuf));
}
	
int get_load(void) {
	char *c;
	char inbuf[128];
	int ret;
	double r;
	FILE *fp;

	if(!(fp = fopen("/proc/loadavg","r")))
		return(0);

	fgets(inbuf,128,fp);
	fclose(fp);

	c = strchr(inbuf,' ');
	strcpy(c,"");
	
	r = strtod(inbuf,NULL);
	ret = r * 100;
	return(ret);
}

int not_overloaded(void) {
	/* eventually this will determine if this machine is too overloaded to handle new IO */
	return(1);
}

char *translate_string(char *string) {
	static char ret[1024];
	int i;
	int j=0;

	for(i=0;i<strlen(string);i++) {
		if(string[i] == '%') {
			i++;
			switch(string[i]) {
				case 'V':
					strcat(ret+j,VERSION);
					j+=strlen(VERSION);
			}
		} else {
			ret[j++] = string[i];
		}
	}

	ret[j] = '\000';
	return(ret);
}

char *get_hostname(void) {
	/* 
	 * get our hostname
	 *
	 */

	static char ret[128];
	FILE *fp;
	char *p;

	if(!(fp = popen("hostname","r"))) 
		return(NULL);
	
	fgets(ret,128,fp);
	
	p = strchr(ret,'.');
	if(p)
		strcpy(p,"");
	return(ret);
}
	
char *get_debug_string(int d) {
	static char ret[1024];

	strcpy(ret,"");

	if(d & DEBUG_CON) strcat(ret,"CON ");
	if(d & DEBUG_CODE) strcat(ret,"CODE ");
	if(d & DEBUG_THREAD) strcat(ret,"THREAD ");
	if(d & DEBUG_IO) strcat(ret,"IO ");
	if(d & DEBUG_HB) strcat(ret,"HB ");
	if(d & DEBUG_IPC) strcat(ret,"IPC ");
	if(d & DEBUG_DB) strcat(ret,"DB ");
	if(d & DEBUG_MAIN) strcat(ret,"MAIN ");
	if(d & DEBUG_PEER) strcat(ret,"PEER ");
	if(d & DEBUG_FILE) strcat(ret,"FILE ");
	if(d & DEBUG_STATION) strcat(ret,"STATION ");
	
	return(ret);
}

int build_debug_flags(struct parse *p) {
	char *ch;
	int ret = 0;
	int i=0;

	if(!p) return(0);
	while((ch = get_token_s(p,i++))) {
		if(!strcasecmp(ch,"all")) 
			ret = 65535;
		if(!strcasecmp(ch,"con"))
			if(!(ret & DEBUG_CON)) ret+= DEBUG_CON;
		if(!strcasecmp(ch,"code")) 
			if(!(ret & DEBUG_CODE)) ret += DEBUG_CODE;
		if(!strcasecmp(ch,"thread"))
			if(!(ret & DEBUG_THREAD)) ret += DEBUG_THREAD;
		if(!strcasecmp(ch,"io"))
			if(!(ret & DEBUG_IO)) ret+= DEBUG_IO;
		if(!strcasecmp(ch,"hb"))
			if(!(ret & DEBUG_HB)) ret+= DEBUG_HB;
		if(!strcasecmp(ch,"ipc"))
			if(!(ret & DEBUG_IPC)) ret += DEBUG_IPC;
		if(!strcasecmp(ch,"db"))
			if(!(ret & DEBUG_DB)) ret += DEBUG_DB;
		if(!strcasecmp(ch,"main"))
			if(!(ret & DEBUG_MAIN)) ret += DEBUG_MAIN;
		if(!strcasecmp(ch,"peer"))
			if(!(ret & DEBUG_PEER)) ret += DEBUG_PEER;
		if(!strcasecmp(ch,"file")) 
			if(!(ret & DEBUG_FILE)) ret += DEBUG_FILE;
		if(!strcasecmp(ch,"station"))
			if(!(ret & DEBUG_STATION)) ret += DEBUG_STATION;
	}

	return(ret);
}

	
	
